package com.ebei.device.asset.weixin.exception;

import com.ebei.device.asset.weixin.constant.CommonErrorCode;
import com.ebei.device.asset.weixin.constant.ResponseConstants;
import com.ebei.device.asset.weixin.util.ResponseEx;
import com.ebei.device.asset.weixin.util.ResponsePageEx;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.NoHandlerFoundException;
import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.List;
import java.util.Set;


/**
 * 全局异常处理控制器
 * @author baihe
 *
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionController {

	@ExceptionHandler(LoginException.class)
	public Object handleAuthorizationException(HttpServletRequest req, LoginException e){
		log.error("["+e.getMessage()+"]",e);
		ResponsePageEx r = ResponsePageEx.createError(ResponseConstants.CODE_UNAUTHORIATION,e.getMessage());
		return new ResponseEntity<>(r, HttpStatus.OK);
	}

	@ExceptionHandler(BaseException.class)
	public Object handleBaseException(HttpServletRequest req, BaseException e){
		log.error("["+e.getMessage()+"]",e);
		ResponsePageEx r = ResponsePageEx.createError(e.getMessage());
		return new ResponseEntity<>(r, HttpStatus.OK);
	}

	@ExceptionHandler(Throwable.class)
	public Object handleThrowable(HttpServletRequest req, Throwable e){
		log.error("["+e.getMessage()+"]",e);
		ResponsePageEx r = ResponsePageEx.createError("系统异常,请联系管理员");
		return new ResponseEntity<>(r, HttpStatus.OK);
	}

	//////////////////////////////////////////以下是新加的////////////////////////////////////////////////

	/**
	 * NoHandlerFoundException 404 异常处理
	 */
	@ExceptionHandler(value = NoHandlerFoundException.class)
	@ResponseStatus(HttpStatus.NOT_FOUND)
	public ResponseEx<Object> handlerNoHandlerFoundException(NoHandlerFoundException exception) {
		outPutErrorWarn(NoHandlerFoundException.class, CommonErrorCode.NOT_FOUND, exception);
		return ResponseEx.createError(CommonErrorCode.NOT_FOUND);
	}

	/**
	 * HttpRequestMethodNotSupportedException 405 异常处理
	 */
	@ExceptionHandler(HttpRequestMethodNotSupportedException.class)
	public ResponseEx<Object> handlerHttpRequestMethodNotSupportedException(
			HttpRequestMethodNotSupportedException exception) {
		outPutErrorWarn(HttpRequestMethodNotSupportedException.class,
				CommonErrorCode.METHOD_NOT_ALLOWED, exception);
		return ResponseEx.createError(CommonErrorCode.METHOD_NOT_ALLOWED);
	}

	/**
	 * HttpMediaTypeNotSupportedException 415 异常处理
	 */
	@ExceptionHandler(HttpMediaTypeNotSupportedException.class)
	public ResponseEx<Object> handlerHttpMediaTypeNotSupportedException(
			HttpMediaTypeNotSupportedException exception) {
		outPutErrorWarn(HttpMediaTypeNotSupportedException.class,
				CommonErrorCode.UNSUPPORTED_MEDIA_TYPE, exception);
		return ResponseEx.createError(CommonErrorCode.UNSUPPORTED_MEDIA_TYPE);
	}

	/**
	 * Exception 类捕获 500 异常处理
	 */
	@ExceptionHandler(value = Exception.class)
	public ResponseEx<Object> handlerException(Exception e) {
		return ifDepthExceptionType(e);
	}

	/**
	 * 二次深度检查错误类型
	 */
	private ResponseEx<Object> ifDepthExceptionType(Throwable throwable) {
		outPutError(Exception.class, CommonErrorCode.EXCEPTION, throwable);
		return ResponseEx.createError(500, throwable.getMessage());
	}


	/**
	 * BusinessException 类捕获
	 */
	@ExceptionHandler(value = BusinessException.class)
	public ResponseEx<Object> handlerBusinessException(BusinessException e) {
		outPutError(BusinessException.class, CommonErrorCode.BUSINESS_ERROR, e);
		return ResponseEx.createError(Integer.valueOf(e.getCode()), e.getMessage());
	}

	/**
	 * HttpMessageNotReadableException 参数错误异常
	 */
	@ExceptionHandler({HttpMessageNotReadableException.class, MissingServletRequestParameterException.class, ServletRequestBindingException.class})
	public ResponseEx<Object> handleHttpMessageNotReadableException(HttpMessageNotReadableException e) {
		outPutError(HttpMessageNotReadableException.class, CommonErrorCode.PARAM_ERROR, e);
		String msg = String.format("%s : 错误详情( %s )", CommonErrorCode.PARAM_ERROR.getMessage(),
				e.getRootCause().getMessage());
		return ResponseEx.createError(Integer.valueOf(CommonErrorCode.PARAM_ERROR.getCode()), msg);
	}

	/**
	 * ConstraintViolationException 参数错误异常
	 */
	@ExceptionHandler(ConstraintViolationException.class)
	public ResponseEx<Object> handleConstraintViolationException(ConstraintViolationException ex) {
		String smg = "";
		Set<ConstraintViolation<?>> constraintViolations = ex.getConstraintViolations();
		if (log.isDebugEnabled()) {
			for (ConstraintViolation error : constraintViolations) {
				log.error("{} -> {}", error.getPropertyPath(), error.getMessageTemplate());
				smg = error.getMessageTemplate();
			}
		}

		if (constraintViolations.isEmpty()) {
			log.error("validExceptionHandler error fieldErrors is empty");
			ResponseEx.createError(Integer.valueOf(CommonErrorCode.BUSINESS_ERROR.getCode()), "");
		}

		return ResponseEx.createError(Integer.valueOf(CommonErrorCode.PARAM_ERROR.getCode()), smg);
	}

	/**
	 * MethodArgumentNotValidException 参数错误异常
	 */
	@ExceptionHandler(MethodArgumentNotValidException.class)
	public ResponseEx<Object> handleMethodArgumentNotValidException(MethodArgumentNotValidException ex) {
		BindingResult bindingResult = ex.getBindingResult();
		return getBindResultDTO(bindingResult);
	}

	/**
	 * BindException 参数错误异常
	 */
	@ExceptionHandler(BindException.class)
	public ResponseEx<Object> handleBindException(BindException e) {
		outPutError(BindException.class, CommonErrorCode.PARAM_ERROR, e);
		BindingResult bindingResult = e.getBindingResult();
		return getBindResultDTO(bindingResult);
	}

	private ResponseEx<Object> getBindResultDTO(BindingResult bindingResult) {
		List<FieldError> fieldErrors = bindingResult.getFieldErrors();
		if (log.isDebugEnabled()) {
			for (FieldError error : fieldErrors) {
				log.error("{} -> {}", error.getDefaultMessage(), error.getDefaultMessage());
			}
		}

		if (fieldErrors.isEmpty()) {
			log.error("validExceptionHandler error fieldErrors is empty");
			ResponseEx.createError(Integer.valueOf(CommonErrorCode.BUSINESS_ERROR.getCode()), "");
		}

		return ResponseEx.createError(Integer.valueOf(CommonErrorCode.PARAM_ERROR.getCode()), fieldErrors.get(0).getDefaultMessage());
	}

	public void outPutError(Class errorType, Enum secondaryErrorType, Throwable throwable) {
		log.error("[{}] {}: {}", errorType.getSimpleName(), secondaryErrorType, throwable.getMessage(), throwable);
	}

	public void outPutErrorWarn(Class errorType, Enum secondaryErrorType, Throwable throwable) {
		log.warn("[{}] {}: {}" , errorType.getSimpleName(), secondaryErrorType, throwable.getMessage());
	}
}
